@@ -19,6 +19,7 @@
  * Boston, MA 02111-1307, USA.
  */
 
+#include "config.h"
 #include <sys/types.h>
 #include <stdlib.h>
 #include <string.h>
@@ -26,10 +27,22 @@
 
 #include <gnome.h>
 
+#ifdef WITH_GNOME_KEYRING
+#include <gnome-keyring.h>
+#endif
+
 #include "seahorse-gconf.h"
 #include "seahorse-gpgmex.h"
 #include "seahorse-agent.h"
 #include "seahorse-secure-memory.h"
+#include "seahorse-key-source.h"
+#include "seahorse-pgp-key.h"
+#include "seahorse-context.h"
+#include "seahorse-unknown-source.h"
+
+/* -----------------------------------------------------------------------------
+ * INTERNAL PASSWORD CACHE
+ */
 
 /*
  * Implementation of the password cache. Note that only passwords
@@ -41,7 +54,7 @@
 
 #define UNPARSEABLE_KEY     _("Unparseable Key ID")
 #define UNKNOWN_KEY         _("Unknown/Invalid Key")
-#define TRANSIENT_ID        "TRANSIENT"
+#define TRANSIENT_ID        "TRANSIENTTRANSIENT"
 
 typedef struct sa_cache_t {
     gchar *id;                  /* The password id */
@@ -53,8 +66,7 @@ typedef struct sa_cache_t {
 
 static GHashTable *g_cache = NULL;      /* Hash of ids to sa_cache_t */
 static GMemChunk *g_memory = NULL;      /* Memory for sa_cache_t's */
-static gpgme_ctx_t g_ctx = NULL;        /* Context for looking up ids */
-static guint g_notify_id = 0;			/* gconf notify id */
+static guint g_notify_id = 0;           /* gconf notify id */
 
 /* -----------------------------------------------------------------------------
  */
@@ -165,16 +177,8 @@ seahorse_agent_cache_init ()
     err = gpgme_engine_check_version (proto);
     g_return_if_fail (GPG_IS_OK (err));
    
-    err = gpgme_new (&g_ctx);
-    g_return_if_fail (GPG_IS_OK (err));
-   
-    err = gpgme_set_protocol (g_ctx, proto);
-    g_return_if_fail (GPG_IS_OK (err));
-   
-    gpgme_set_keylist_mode (g_ctx, GPGME_KEYLIST_MODE_LOCAL);
-	
-	/* Listen for changes on the AUTH key */
-	g_notify_id = seahorse_gconf_notify (SETTING_AUTH, gconf_notify, NULL);
+    /* Listen for changes on the AUTH key */
+    g_notify_id = seahorse_gconf_notify (SETTING_AUTH, gconf_notify, NULL);
 }
 
 /* Uninitialize and free up cache memory */
@@ -193,20 +197,15 @@ seahorse_agent_cache_uninit ()
         g_memory = NULL;
     }
     
-    if (g_ctx) {
-        gpgme_release (g_ctx);
-        g_ctx = NULL;
+    if (g_notify_id) {
+        seahorse_gconf_unnotify (g_notify_id);
+        g_notify_id = 0;
     }
-	
-	if (g_notify_id) {
-		seahorse_gconf_unnotify (g_notify_id);
-		g_notify_id = 0;
-	}
 }
 
 /* Retrieve a password from the cache */
 const gchar *
-seahorse_agent_cache_get (const gchar *id)
+seahorse_agent_internal_get (const gchar *id)
 {
     sa_cache_t *it;
 
@@ -235,7 +234,7 @@ seahorse_agent_cache_get (const gchar *id)
 
 /* Check if a given id is in the cache, and lock if requested */
 gboolean
-seahorse_agent_cache_has (const gchar *id, gboolean lock)
+seahorse_agent_internal_has (const gchar *id, gboolean lock)
 {
     sa_cache_t *it;
 
@@ -259,7 +258,7 @@ seahorse_agent_cache_has (const gchar *id, gboolean lock)
 
 /* Remove given id from the cache */
 void
-seahorse_agent_cache_clear (const gchar *id)
+seahorse_agent_internal_clear (const gchar *id)
 {
     if (id == NULL)
         id = TRANSIENT_ID;
@@ -283,7 +282,7 @@ remove_cache_item (gpointer key, gpointer value, gpointer user_data)
      * This is a simple callback for removing all 
      * items from a GHashTable. returning TRUE removes. 
      */
-	return !it->locked;
+    return !it->locked;
 }
 
 /* Clear all items in the cache */
@@ -296,32 +295,11 @@ seahorse_agent_cache_clearall ()
         seahorse_agent_status_update ();
 }
 
-/* Encode a password in hex */
-static void
-encode_password (gchar *dest, const gchar *src)
-{
-    static const char HEXC[] = "0123456789abcdef";
-    int j;
-
-    g_assert (dest && src);
-
-    /* Simple hex encoding */
-
-    while (*src) {
-        j = *(src) >> 4 & 0xf;
-        *(dest++) = HEXC[j];
-
-        j = *(src++) & 0xf;
-        *(dest++) = HEXC[j];
-    }
-}
 
 /* Set a password in the cache. encode and lock if requested */
 void
-seahorse_agent_cache_set (const gchar *id, const gchar *pass,
-                          gboolean encode, gboolean lock)
+seahorse_agent_internal_set (const gchar *id, const gchar *pass, gboolean lock)
 {
-    int len, c;
     gboolean cache;
     sa_cache_t *it;
 
@@ -351,33 +329,10 @@ seahorse_agent_cache_set (const gchar *id, const gchar *pass,
     g_assert (it->id != NULL);
 
     /* Work with the password */
-
     if (it->pass)
         seahorse_secure_memory_free (it->pass);
-
-    len = strlen (pass);
-
-    if (encode) {
-        c = sizeof (gchar *) * ((len * 2) + 1);
-        it->pass = (gchar *) seahorse_secure_memory_malloc (c);
-        if (!it->pass) {
-            g_critical ("out of secure memory");
-            return;
-        }
-
-        memset (it->pass, 0, c);
-        encode_password (it->pass, pass);
-    }
-
-    else {
-        it->pass = (gchar *) seahorse_secure_memory_malloc (sizeof (gchar) * (len + 1));
-        if (!it->pass) {
-            g_critical ("out of secure memory");
-            return;
-        }
-
-        strcpy (it->pass, pass);
-    }
+    it->pass = seahorse_secure_memory_malloc (strlen (pass) + 1);
+    strcpy (it->pass, pass);
 
     /* If not caching set to the epoch which should always expire */
     it->stamp = cache ? time (NULL) : 0;
@@ -397,43 +352,239 @@ seahorse_agent_cache_count ()
     return g_hash_table_size (g_cache);
 }
 
-/* For enumerating through all ids in the cache */
-void
-seahorse_agent_cache_enum (GHFunc func, gpointer user_data)
+static void 
+build_key_list (gpointer key, gpointer value, GList **keys)
 {
-    g_assert (g_cache != NULL);
-    g_assert (func != NULL);
-    g_hash_table_foreach (g_cache, func, user_data);
+    sa_cache_t *it = (sa_cache_t*)value;
+    SeahorseKeySource *sksrc;
+    SeahorseKey *skey;
+    GQuark keyid;
+    
+    keyid = seahorse_pgp_key_get_cannonical_id (it->id);
+    g_return_if_fail (keyid);
+
+    skey = seahorse_context_find_key (SCTX_APP (), keyid, SKEY_LOC_LOCAL);
+    if (!skey) {
+        sksrc = seahorse_context_find_key_source (SCTX_APP (), SKEY_PGP, SKEY_LOC_UNKNOWN);
+        g_return_if_fail (sksrc != NULL);
+        skey = seahorse_unknown_source_add_key (SEAHORSE_UNKNOWN_SOURCE (sksrc), keyid, NULL);
+    }
+    
+    if (skey) 
+        *keys = g_list_prepend (*keys, skey);
 }
 
-/* Get the displayable name of a given cache item */
-gchar *
-seahorse_agent_cache_getname (const gchar *id)
+/* Get list of all SeahorseKey's cached */
+GList* 
+seahorse_agent_cache_get_keys ()
 {
-    gpgme_key_t key;
-    gchar *userid;
-    
-    if (!g_ctx)
-        return g_strdup (UNKNOWN_KEY);
+    GList *keys = NULL;
+    g_assert (g_cache != NULL);
+    g_hash_table_foreach (g_cache, (GHFunc)build_key_list, &keys);
+    return keys;
+}
 
-    g_return_val_if_fail (g_ctx != NULL, g_strdup (UNKNOWN_KEY));
-    g_return_val_if_fail (id != NULL, g_strdup (UNKNOWN_KEY));
+
+/* -----------------------------------------------------------------------------
+ * GENERIC CACHE FUNCTIONS
+ */
+
+#define KEYRING_ATTR_TYPE "seahorse-key-type"
+#define KEYRING_ATTR_KEYID "openpgp-keyid"
+#define KEYRING_VAL_GPG "openpgp"
+
+#ifdef WITH_GNOME_KEYRING
+
+static gboolean 
+only_internal_cache ()
+{
+    gboolean internal = TRUE;
+    
+    /* No cache, internal must still work though */
+    if (!seahorse_gconf_get_boolean (SETTING_CACHE))
+        internal = TRUE;
+    
+    else {
+        gchar *method = seahorse_gconf_get_string (SETTING_METHOD);
+        if (method && strcmp (method, METHOD_GNOME) == 0)
+            internal = FALSE;
+        g_free (method);
+    }
+    
+    return internal;
+}
+
+static const gchar*
+gkeyring_keyid (const gchar *fullid)
+{
+    size_t len = strlen(fullid);
+    return fullid + (len > 16 ? len - 16 : 0);
+}
+
+#endif /* WITH_GNOME_KEYRING */
+    
+void
+seahorse_agent_cache_set (const gchar *id, const gchar *pass, gboolean lock)
+{
+    /* Store in our internal cache */
+    seahorse_agent_internal_set (id, pass, lock);
+
+#ifdef WITH_GNOME_KEYRING
+    
+    /* Store in gnome-keyring */
+    if (id && !only_internal_cache ()) {
         
-    gpgme_get_key (g_ctx, id, &key, 1);
-    if (key == NULL) {
-        gpgme_get_key (g_ctx, id, &key, 0);
-        if (key == NULL) 
-            return g_strdup (UNKNOWN_KEY);
+        GnomeKeyringResult res;
+        GnomeKeyringAttributeList *attributes = NULL;
+        guint item_id;
+        gchar *desc = NULL, *name;
+        SeahorseKey *skey;
+        GQuark keyid;
+        
+        keyid = seahorse_pgp_key_get_cannonical_id (id);
+        skey = seahorse_context_find_key (SCTX_APP (), keyid, SKEY_LOC_LOCAL);
+        
+        if (skey) {
+            name = seahorse_key_get_display_name (skey);
+            desc = g_strdup_printf (_("PGP Key: %s"), name);
+            g_free (name);
+        }
+        
+        attributes = gnome_keyring_attribute_list_new ();
+        gnome_keyring_attribute_list_append_string (attributes, KEYRING_ATTR_TYPE, 
+                                                    KEYRING_VAL_GPG);
+        gnome_keyring_attribute_list_append_string (attributes, KEYRING_ATTR_KEYID, 
+                                                    gkeyring_keyid (id));
+        res = gnome_keyring_item_create_sync (NULL, GNOME_KEYRING_ITEM_GENERIC_SECRET, 
+                                              desc, attributes, pass, TRUE, &item_id);
+        gnome_keyring_attribute_list_free (attributes);
+        g_free (desc);
+        
+        if (res != GNOME_KEYRING_RESULT_OK)
+            g_warning ("Couldn't store password in keyring: (code %d)", res);
+    }
+    
+#endif /* WITH_GNOME_KEYRING */
+    
+}
+
+void
+seahorse_agent_cache_clear (const gchar *id)
+{
+    /* Clear from our internal cache */
+    seahorse_agent_internal_clear (id);
+    
+#ifdef WITH_GNOME_KEYRING 
+    
+    /* Clear from gnome-keyring */
+    if (id && !only_internal_cache ()) {
+        
+        GnomeKeyringResult res;
+        GnomeKeyringAttributeList *attributes = NULL;
+        GList *found_items, *l;
+        
+        attributes = gnome_keyring_attribute_list_new ();
+        gnome_keyring_attribute_list_append_string (attributes, KEYRING_ATTR_KEYID, 
+                                                    gkeyring_keyid (id));
+        res = gnome_keyring_find_items_sync (GNOME_KEYRING_ITEM_GENERIC_SECRET,
+                                             attributes, &found_items);
+        gnome_keyring_attribute_list_free (attributes);
+        
+        if (res != GNOME_KEYRING_RESULT_OK) {
+            g_warning ("couldn't search keyring: (code %d)", res);
+            
+        } else {
+            for (l = found_items; l; l = g_list_next (l)) {
+                /* TODO: Can we use async here? */
+                res = gnome_keyring_item_delete_sync (NULL, 
+                                    ((GnomeKeyringFound*)(l->data))->item_id);
+                if (res != GNOME_KEYRING_RESULT_OK)
+                    g_warning ("Couldn't clear password from keyring: (code %d)", res);
+            }
+            
+            gnome_keyring_found_list_free (found_items);
+        }
+        
+    }
+    
+#endif /* WITH_GNOME_KEYRING */
+
+}
+
+const gchar* 
+seahorse_agent_cache_get (const gchar *id)
+{
+    const gchar *ret = NULL;
+    
+    /* Always look in our own keyring first */
+    ret = seahorse_agent_internal_get (id);
+        
+#ifdef WITH_GNOME_KEYRING 
+    
+    /* Clear from gnome-keyring */
+    if (!ret && id && !only_internal_cache ()) {
+        
+        GnomeKeyringResult res;
+        GnomeKeyringAttributeList *attributes = NULL;
+        GList *found_items;
+        GnomeKeyringFound *found;
+        
+        attributes = gnome_keyring_attribute_list_new ();
+        gnome_keyring_attribute_list_append_string (attributes, KEYRING_ATTR_KEYID, 
+                                                    gkeyring_keyid (id));
+        res = gnome_keyring_find_items_sync (GNOME_KEYRING_ITEM_GENERIC_SECRET,
+                                             attributes, &found_items);
+        gnome_keyring_attribute_list_free (attributes);
+        
+        if (res != GNOME_KEYRING_RESULT_OK) {
+            if (res != GNOME_KEYRING_RESULT_DENIED)
+                g_warning ("couldn't search keyring: (code %d)", res);
+            
+        } else {
+            
+            if (found_items && found_items->data) {
+                found = (GnomeKeyringFound*)found_items->data;
+                if (found->secret) {
+                    
+                    /* Store it temporarily in our loving hands */
+                    seahorse_agent_internal_set (NULL, found->secret, TRUE);
+                    ret = seahorse_agent_internal_get (NULL);
+                
+                }
+            }
+            
+            gnome_keyring_found_list_free (found_items);
+        }
+        
+    }
+    
+#endif /* WITH_GNOME_KEYRING */
+    
+    return ret;
+}
+
+/* Check if a given id is in the cache, and lock if requested */
+gboolean
+seahorse_agent_cache_has (const gchar *id, gboolean lock)
+{
+    if (seahorse_agent_internal_has (id, lock))
+        return TRUE;
+    
+#ifdef WITH_GNOME_KEYRING 
+
+    /* Retrieve from keyring and lock in local */
+    if (id && !only_internal_cache ()) {
+        
+        const gchar *pass = seahorse_agent_cache_get (id);
+        if (!pass)
+            return FALSE;
+        
+        /* Store it in our loving hands */
+        seahorse_agent_internal_set (id, pass, TRUE);
+        return TRUE;
     }
 
-    userid = (key->uids && key->uids->uid) ? key->uids->uid : UNKNOWN_KEY;
+#endif /* WITH_GNOME_KEYRING */
     
-    /* If not utf8 valid, assume latin 1 */
-    if (!g_utf8_validate (userid, -1, NULL))
-        userid = g_convert (userid, -1, "UTF-8", "ISO-8859-1", NULL, NULL, NULL);
-    else
-        userid = g_strdup (userid);
-
-    gpgme_key_unref (key);
-    return userid;
+    return FALSE;
 }
