@@ -153,8 +153,8 @@ seahorse_agent_prompt_pass (SeahorseAgentPassReq *pr)
 
     g_assert (!seahorse_agent_prompt_have ());
     
-    dialog = seahorse_passphrase_prompt_show (NULL, pr->description, 
-                                              pr->prompt, pr->errmsg);
+    dialog = seahorse_passphrase_prompt_show (NULL, pr->errmsg ? pr->errmsg : pr->description, 
+                                              pr->prompt, NULL, FALSE);
     g_signal_connect (dialog, "response", G_CALLBACK (passphrase_response), pr);
     g_current_win = GTK_WIDGET (dialog);
 }
@@ -213,7 +213,7 @@ create_auth_window (SeahorseAgentPassReq *pr)
     gchar *msg;
 
     win = gtk_window_new (GTK_WINDOW_TOPLEVEL);
-    gtk_window_set_title (GTK_WINDOW (win), _("Authorize Password Access"));
+    gtk_window_set_title (GTK_WINDOW (win), _("Authorize Passphrase Access"));
     acc = gtk_accel_group_new ();
 
     g_signal_connect (G_OBJECT (win), "delete_event",
